<?php
/**
 * @package phpBB3
 * @version 1.0.4
 * @ copyright (c) 2009 RMcGirr83
 * @copyright (c) 2007 eviL3
 * @license http://opensource.org/licenses/gpl-license.php GNU Public License(
 */
/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}
/**
 * Install the contact form
 */
function install_contact($contact_version)
{
	global $phpbb_root_path, $phpEx;
	global $user;
	
	if ($user->data['user_type'] != USER_FOUNDER)
	{
		trigger_error('NOT_AUTHORISED');
	}
	
	set_config('contact_enable', 0);
	set_config('contact_confirm', 1);
	set_config('contact_confirm_guests', 1);
	set_config('contact_max_attempts', 0);
	set_config('contact_method', 0);
	set_config('contact_bot_user', 2);
	set_config('contact_bot_forum', 2);
	set_config('contact_reasons', '');
	set_config('contact_founder_only', 0);
	set_config('contact_bbcodes_allowed', 0);
	set_config('contact_smilies_allowed', 0);
	set_config('contact_bot_poster', 0);
	
	set_config('contact_version', $contact_version, false);
	
	// confirm message
	trigger_error($user->lang['CONTACT_INSTALLED'] . '<br /><br />' . sprintf($user->lang['RETURN_CONTACT'], '<a href="' . append_sid("{$phpbb_root_path}contact.$phpEx") . '">', '</a>'));
}

/**
 * Update the contact form
 * 
 * Add each update entry in a separate if () clause, for example:
 * <code>
 * 	if ($config['contact_version'] < '0.1.4')
 * 	{
 * 		$db->sql_query($some_query);
 * 	}
 * </code>
 */
function update_contact($contact_version)
{
	global $phpbb_root_path, $phpEx, $config;
	global $user, $cache;
	
	if ($user->data['user_type'] != USER_FOUNDER)
	{
		set_config('contact_enable', 0);
		
		$message = sprintf($user->lang['CONTACT_MAIL_DISABLED'], '<a href="mailto:' . $config['board_contact'] . '">', '</a>');
		// show message that contact form is disabled
		trigger_error($message);
	}
	
	if (version_compare($config['contact_version'], '1.0.0', '<'))
	{
		set_config('contact_founder_only', 0);
	}
	if (version_compare($config['contact_version'], '1.0.2', '<'))
	{
		set_config('contact_bbcodes_allowed', 0);
		set_config('contact_smilies_allowed', 0);
	}
	if (version_compare($config['contact_version'], '1.0.4', '<'))
	{
		set_config('contact_bot_poster', 0);
	}	
	// update config table with new version
	set_config('contact_version', $contact_version, false);
	
	$cache->purge();
	
	// confirm message
	trigger_error(sprintf($user->lang['CONTACT_UPDATED'], $contact_version) . '<br /><br />' . sprintf($user->lang['RETURN_CONTACT'], '<a href="' . append_sid("{$phpbb_root_path}contact.$phpEx") . '">', '</a>'));
}

/**
 * Uninstall contact board admin
 */
function uninstall_contact()
{
	global $db, $cache;
	
	$sql = 'DELETE FROM ' . CONFIG_TABLE . ' 
		WHERE ' . $db->sql_in_set('config_name', array('contact_enable', 'contact_confirm', 'contact_confirm_guests', 'contact_max_attempts', 'contact_method', 'contact_bot_user', 'contact_bot_forum', 'contact_version', 'contact_reasons', 'contact_founder_only', 'contact_bbcodes_allowed', 'contact_smilies_allowed', 'contact_bot_poster'));
	$db->sql_query($sql);
	
	$cache->purge();
	
	// confirm message
	trigger_error($user->lang['CONTACT_UNINSTALLED']);
}
/**
* contact_change_auth
* thanks to poppertom69 for the idea..and some of the code
* 
*/
function contact_change_auth($bot_id, $mode = 'replace', $bkup_data = false)
{
	global $auth, $db, $config, $user;
	
	switch($mode)
	{
		// we want auths for the user making the contact 
		// and a post count high enough in case queing is enabled
		case 'replace':
			
			// admin doesn't want posts/pms to come from bot
			// selected in ACP
			if ($config['contact_bot_poster'] == CONTACT_POST_NEITHER || ($config['contact_bot_poster'] == CONTACT_POST_GUEST && $user->data['is_registered']))
			{
				$bkup_data = array(
					'user_bkup_posts'	=> $user->data['user_posts'],
					'user_bkup_reg'		=> $user->data['is_registered'],
					'auth_backup'		=> $auth,
				);
				
				// sql to get the bots info
				$sql = 'SELECT *
					FROM ' . USERS_TABLE . "
					WHERE user_id = '" . $db->sql_escape((int) $bot_id) . "'";
				$result	= $db->sql_query($sql);
				$row = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);
					
				// reset the current users auths to those of the bot	
				$auth_user->data = $row;
				$auth->acl($auth_user->data);
			}
			// admin only wants bot for guests
			elseif ($config['contact_bot_poster'] == CONTACT_POST_GUEST && !$user->data['is_registered'])
			{
				$bkup_data = array(
					'user_backup'	=> $user->data,
					'auth_backup'	=> $auth,
				);
				
				// sql to get the bots info
				$sql = 'SELECT *
					FROM ' . USERS_TABLE . "
					WHERE user_id = '" . $db->sql_escape((int) $bot_id) . "'";
				$result	= $db->sql_query($sql);
				$row = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);
					
				// reset the current users auths to those of the bot	
				$user->data = array_merge($user->data, $row);
				$auth->acl($user->data);
			}
			else // admin wants the bot to show as the poster for everyone
			{
				$bkup_data = array(
					'user_backup' => $user->data,
					'auth_backup' => $auth,
				);

				$sql = 'SELECT *
					FROM ' . USERS_TABLE . "
					WHERE user_id = '" . $db->sql_escape((int) $bot_id) . "'";
				$result	= $db->sql_query($sql);
				$row = $db->sql_fetchrow($result);
				
				$db->sql_freeresult($result);

				$user->data = array_merge($user->data, $row);
				$auth->acl($user->data);
			}
			
			// check for queing enabled
			// if the bot doesn't have a high enough post count...make it high enough
			if ($config['enable_queue_trigger'] && $user->data['user_posts'] < $config['queue_trigger_posts'])
			{
				$user->data['user_posts'] = (int) $config['queue_trigger_posts'];
			}
					
			unset($row);
			
			return $bkup_data;					
			
		break;
		
		// now we restore the auths and post count from the backup array
		case 'restore':

			if (isset($bkup_data['user_bkup_reg']))
			{
				$user->data['user_posts'] = $bkup_data['user_bkup_posts'];
				$user->data['is_registered'] = $bkup_data['user_bkup_reg'];
				$auth = $bkup_data['auth_backup'];
			}
			else
			{
				$user->data = $bkup_data['user_backup'];
				$auth = $bkup_data['auth_backup'];
			}
			unset($bkup_data);
		break;
	}
}
/**
* contact_check
* @param $forum_id, the forum id selected in ACP
* @param $forum_name returned from contact_check_forum
* ensures postable forum and correctly authed "bot"
*/
function contact_check($forum_id = false, $mode, $forum_name = '')
{
	global $phpbb_root_path, $phpEx, $config;
	global $auth, $db, $user;
	
	// the servers url 
	$server_url = generate_board_url();	
	
	switch($mode)
	{
		// check for a valid bot..we need correct auths
		case 'contact_check_bot':
			
			// forum_id passed?
			$forum_id = (!empty($forum_id)) ? (int) $forum_id : '';
						
			// an array of users we ignore
			// could add USER_INACTIVE but doesn't really matter
			$ignored_users = array(USER_IGNORE);
	
			$sql = 'SELECT username, user_id
				FROM ' . USERS_TABLE . '
					WHERE user_id = ' . (int) $config['contact_bot_user'] . '
				AND ' . $db->sql_in_set('user_type', $ignored_users, true);
			$result = $db->sql_query($sql);
			$row = $db->sql_fetchrow($result);
			$db->sql_freeresult($result);
			
			// we didn't get a result
			// send an email if board enabled
			if (!$row && $config['email_enable'])
			{
				// send an email to the board default
				$email_template = 'contact_error_bot';
				$email_message = sprintf($user->lang['CONTACT_BOT_USER_MESSAGE'], $user->data['username'], $config['sitename'], $server_url);
				contact_send_email($email_template, $email_message);
				
				// disable the contact mod
				set_config('contact_enable', 0);
				
				// add an entry into the log error table
				add_log('critical', 'LOG_CONTACT_BOT_INVALID', $config['contact_bot_user']);
	
				$meta_info = append_sid("{$phpbb_root_path}index.$phpEx");
				meta_refresh(5, $meta_info);

				// show a message to the user
				$message = $user->lang['CONTACT_BOT_ERROR'] . '<br /><br />' . sprintf($user->lang['RETURN_INDEX'], '<a href="' . append_sid("{$phpbb_root_path}index.$phpEx") . '">', '</a>');
				trigger_error($message);		
			}
			elseif (!$row)
			{
				// disable the contact mod
				set_config('contact_enable', 0);
				
				// add an entry into the log error table
				add_log('critical', 'LOG_CONTACT_BOT_INVALID', $config['contact_bot_user']);
				
				// show a message to the user
				$message = sprintf($user->lang['CONTACT_MAIL_DISABLED'], '<a href="mailto:' . $config['board_contact'] . '">', '</a>');
				trigger_error($message);
			}
			else
			{
				// let's check the bots auths
				//a few variables that we need
				$contact_message = '';
				$contact_bot_authed = true;
							
				// now check the forum, if given, for the contact bot permissions
				// need f_noapprove
				// to avoid placing contact info into the moderation queue if activated
				if ($forum_id && $config['contact_method'] == CONTACT_METHOD_POST)
				{
					// change the users auths momentarily
					$contact_perms = contact_change_auth($row['user_id']);
					
					// teh auth check
					if (!$auth->acl_get('f_noapprove', $forum_id))
					{
						$contact_bot_authed = false;
						$contact_message = sprintf($user->lang['CONTACT_BOT_NO_FORUM_AUTHS_APPROVE'], $row['username'], $config['sitename'], $forum_name, $server_url);
					}
				
					// revert the auths that the user originally had
					contact_change_auth('', 'restore', $contact_perms);
				}
								
				// we didn't get the proper auths
				// send an email if board enabled
				if (!$contact_bot_authed && $config['email_enable'])
				{
					// send an email to the board default
					$email_template = 'contact_error_user';
					$email_message = $contact_message;
					contact_send_email($email_template, $email_message);				
					
					// disable the contact mod
					set_config('contact_enable', 0);
					
					// add an entry into the log error table
					add_log('critical', 'LOG_CONTACT_BOT_INVALID', $row['username']);
					
					$meta_info = append_sid("{$phpbb_root_path}index.$phpEx");
					meta_refresh(5, $meta_info);
					
					// show a message to the user
					$message = $user->lang['CONTACT_BOT_ERROR'] . '<br /><br />' . sprintf($user->lang['RETURN_INDEX'], '<a href="' . append_sid("{$phpbb_root_path}index.$phpEx") . '">', '</a>');
					trigger_error($message);		
				}
				elseif (!$contact_bot_authed)
				{
					// disable the contact mod
					set_config('contact_enable', 0);
					
					// add an entry into the log error table
					add_log('critical', 'LOG_CONTACT_BOT_INVALID', $row['username']);
					
					$message = sprintf($user->lang['CONTACT_MAIL_DISABLED'], '<a href="mailto:' . $config['board_contact'] . '">', '</a>');
					// show message that contact form is disabled
					trigger_error($message);
				}				
			}
		break;
		
		// check for a valid forum
		case 'contact_check_forum':				

			$sql = 'SELECT forum_name
			FROM ' . FORUMS_TABLE . '
			WHERE forum_id = ' . (int) $forum_id . '
			AND forum_type = ' . FORUM_POST;
			$result = $db->sql_query($sql);	
	
			$row = $db->sql_fetchrow($result);
			$db->sql_freeresult($result);
					
			// we didn't get a result
			// send an email if board enabled					
			if (!$row && $config['email_enable'])
			{
				// send an email to the board default
				$email_template = 'contact_error_forum';
				$email_message = sprintf($user->lang['CONTACT_BOT_FORUM_MESSAGE'], $user->data['username'], $config['sitename'], $server_url);
				contact_send_email($email_template, $email_message);
				
				// disable the contact mod
				set_config('contact_enable', 0);
						
				// add an entry into the error log
				add_log('critical', 'LOG_CONTACT_FORUM_INVALID', $config['contact_bot_forum']);		

				$meta_info = append_sid("{$phpbb_root_path}index.$phpEx");
				meta_refresh(5, $meta_info);
				
				// show a message to the user
				$message = $user->lang['CONTACT_BOT_ERROR'] . '<br /><br />' . sprintf($user->lang['RETURN_INDEX'], '<a href="' . append_sid("{$phpbb_root_path}index.$phpEx") . '">', '</a>');
				trigger_error($message);		
			}
			elseif (!$row)
			{
				// disable the contact mod
				set_config('contact_enable', 0);
						
				// add an entry into the error log
				add_log('critical', 'LOG_CONTACT_FORUM_INVALID', $config['contact_bot_forum']);
					
				$message = sprintf($user->lang['CONTACT_MAIL_DISABLED'], '<a href="mailto:' . $config['board_contact'] . '">', '</a>');
				// show message that contact form is disabled
				trigger_error($message);				
			}
			else
			{
				// check the bot..has auths for the forum selected in the ACP
				contact_check($forum_id, 'contact_check_bot', $row['forum_name']);
			}
		break;
			
		case 'contact_nobody':
		
			//this is only called if there are no contact admins available
			// for pm'ing or for emailing to per the preferences set by the admin user in their profiles
					
			if ($config['contact_method'] == CONTACT_METHOD_EMAIL)
			{
				$error = $user->lang['CONTACT_METHOD_EMAIL'];
			}
			else
			{
				$error = $user->lang['CONTACT_METHOD_PM'];
			}
			// only send an email if the board allows it
			if ($config['email_enable'])
			{				
				// send an email to the board default
				$email_template = 'contact_error_user';
				$email_message = sprintf($user->lang['CONTACT_BOT_NONE'], $user->data['username'], $config['sitename'], $error, $server_url);
				contact_send_email($email_template, $email_message);
								
				// disable the contact mod
				set_config('contact_enable', 0);	
				
				// add an entry into the log error table
				add_log('critical', 'LOG_CONTACT_NONE', $error);

				$meta_info = append_sid("{$phpbb_root_path}index.$phpEx");
				meta_refresh(5, $meta_info);			
				// show a message to the user
				$message = $user->lang['CONTACT_BOT_ERROR'] . '<br /><br />' . sprintf($user->lang['RETURN_INDEX'], '<a href="' . append_sid("{$phpbb_root_path}index.$phpEx") . '">', '</a>');
				trigger_error($message);
			}
			else
			{
				// disable the contact mod
				set_config('contact_enable', 0);
						
				// add an entry into the log error table
				add_log('critical', 'LOG_CONTACT_NONE', $error);
					
				$message = sprintf($user->lang['CONTACT_MAIL_DISABLED'], '<a href="mailto:' . $config['board_contact'] . '">', '</a>');
				// show message that contact form is disabled
				trigger_error($message);
			}			
		break;
	}
	return;
}
/**
 * contact_send_email
 * @param $email_template, the email template to use
 * @param $email_message, the message we are sending
 * sends an email to the board default if an error occurs
 */
function contact_send_email($email_template, $email_message)
{
	global $phpbb_root_path, $phpEx;
	global $user, $config;
	
	if (!function_exists('send'))
	{
		include($phpbb_root_path . 'includes/functions_messenger.' . $phpEx);
	}
				
	$messenger = new messenger(false);
			
	// Email headers
	$messenger->headers('X-AntiAbuse: Board servername - ' . $config['server_name']);
	$messenger->headers('X-AntiAbuse: User_id - ' . $user->data['user_id']);
	$messenger->headers('X-AntiAbuse: Username - ' . $user->data['username']);
	$messenger->headers('X-AntiAbuse: User IP - ' . $user->ip);
				
	$messenger->template($email_template, $config['default_lang']);
	$messenger->to($config['board_email']);
	$messenger->from($config['server_name']);

	$messenger->assign_vars(array(
		'SUBJECT'		=> $user->lang['CONTACT_BOT_SUBJECT'],
		'EMAIL_SIG'  	=> $config['board_email_sig'],
		'MESSAGE'		=> $email_message,
	));
	
	$messenger->send(NOTIFY_EMAIL);
	
	return;
}

/**
 * contact_make_select
 * 
 * @param array $input_ary
 * @return string Select html
 * for drop down reasons
 */
function contact_make_select($input_ary, $selected)
{
	// only accept arrays, no empty ones
	if (!is_array($input_ary) || !sizeof($input_ary))
	{
		return;
	}
	
	// If selected isn't in the array, use first entry
	if (!in_array($selected, $input_ary))
	{
		$selected = $input_ary[0];
	}
	
	$select = '';
	foreach ($input_ary as $item)
	{
		//$item = htmlspecialchars($item);
		$item_selected = ($item == $selected) ? ' selected="selected"' : '';
		$select .= '<option value="' . $item . '"' . $item_selected . '>' . $item . '</option>';
	}
	return $select;
}

// ACP Functions
/**
 * make_user_select
 * @return string User List html
 * for drop down when selecting the contact bot
 */
function make_user_select($select_id = false)
{
	global $db, $auth;
	
	// an array of users we ignore
	// could add USER_INACTIVE but doesn't really matter
	$ignored_users = array(USER_IGNORE);	
	
	// variables
	$sql_or = $user_list = '';
	// grab all admins
	
	$admin_arry = $auth->acl_get_list(false, 'a_', false);
	$admin_arry = (!empty($admin_arry[0]['a_'])) ? $admin_arry[0]['a_'] : array();	
	
	if(sizeof($admin_arry))
	{
		$sql_or .= ' OR ' . $db->sql_in_set('user_id', $admin_arry);
	}
	
	// do the main sql query
	$sql = 'SELECT user_id, username
		FROM ' . USERS_TABLE . '
		WHERE (user_type = ' . USER_FOUNDER . '
			' . $sql_or . ') 
			AND ' . $db->sql_in_set('user_type', $ignored_users, true) . '
		ORDER BY username';
	$result = $db->sql_query($sql);

	while ($row = $db->sql_fetchrow($result))
	{	
		$selected = ($row['user_id'] == $select_id) ? ' selected="selected"' : '';
		$user_list .= '<option value="' . $row['user_id'] . '"' . $selected . '>' . $row['username'] . '</option>';
	}
	$db->sql_freeresult($result);

	return $user_list;	
}

?>